"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KnapsackAngularRenderer = void 0;
const app_1 = require("@knapsack/app");
const file_utils_1 = require("@knapsack/file-utils");
const handlebars_helpers_1 = require("@knapsack/app/dist/lib/handlebars-helpers");
const utils_1 = require("@knapsack/utils");
const chokidar_1 = __importDefault(require("chokidar"));
const fs_extra_1 = require("fs-extra");
const path_1 = require("path");
// Need to import this way https://github.com/cheeriojs/cheerio/releases/tag/v1.0.0-rc.9
// eslint-disable-next-line @typescript-eslint/no-var-requires
const cheerio = require('cheerio').default;
const pkg = JSON.parse((0, fs_extra_1.readFileSync)((0, path_1.join)(__dirname, '../package.json'), 'utf-8'));
app_1.log.setupUpdateNotifier(pkg);
const rendererId = 'angular';
const createNgTemplate = (0, handlebars_helpers_1.hbsCompile)((0, fs_extra_1.readFileSync)((0, path_1.join)(__dirname, '../ng-template.ts.hbs'), 'utf-8'));
class KnapsackAngularRenderer extends app_1.KnapsackRendererBase {
    constructor() {
        super({
            id: rendererId,
            language: 'ts',
            extension: 'ts',
        });
        this.getMeta = () => ({
            id: this.id,
            title: 'Angular',
            aliasUse: 'required',
            aliasTitle: 'Named Export',
            aliasDescription: 'JS export name',
            enableDataDemos: true,
            enableTemplateDemos: false,
            hasSlotsSupport: true,
            aliasIsJsNamedExport: true,
            version: pkg.version,
            hasInferSpecSupport: false,
            hasTemplateSuggestionsSupport: true,
        });
        try {
            this.angularJson = JSON.parse((0, fs_extra_1.readFileSync)((0, path_1.join)(process.cwd(), './angular.json'), 'utf-8'));
        }
        catch (e) {
            app_1.log.error(`Could not read "angular.json" in your current working directory`);
        }
        const project = this.angularJson.projects['knapsack-renderer-angular-app'];
        if (!project) {
            throw new Error(`angular.json should have a project "knapsack-renderer-angular-app"`);
        }
        this.ngAppRoot = project.root;
        this.extraHtmlTags = [];
    }
    writeFile() {
        const distDir = (0, path_1.join)(this.ngAppRoot, './src/dist');
        (0, fs_extra_1.ensureDirSync)(distDir);
        const { patterns } = this.getMyTemplates();
        const data = {
            imports: [],
            templates: [],
        };
        patterns.forEach((p) => {
            p.templates.forEach(({ id: templateId, alias: namedExport }) => {
                const { type, originalPath, absolutePath, exists } = this.patterns.getTemplatePath({
                    patternId: p.id,
                    templateId,
                });
                if (!exists) {
                    throw new Error(`Template "${originalPath}" does not exist`);
                }
                let importPath = '';
                if (type !== 'package') {
                    const { dir, name } = (0, path_1.parse)(absolutePath);
                    importPath = (0, path_1.relative)(distDir, (0, path_1.format)({
                        dir,
                        name,
                    }));
                }
                else {
                    importPath = originalPath;
                }
                const theImport = data.imports.find((i) => i.path === importPath);
                if (theImport) {
                    theImport.namedExports.push(namedExport);
                }
                else {
                    data.imports.push({
                        path: importPath,
                        namedExports: [namedExport],
                    });
                }
                data.templates.push({
                    patternId: p.id,
                    templateId,
                    component: namedExport,
                });
            });
        });
        const ngTemplate = app_1.KnapsackRendererBase.formatCode({
            code: createNgTemplate(data),
            language: 'ts',
        });
        (0, fs_extra_1.writeFileSync)((0, path_1.join)(distDir, 'ks-ng-app.module.ts'), ngTemplate, 'utf-8');
    }
    async init(opt) {
        super.init(opt);
        this.ksPublicPath = opt.config.public;
        this.config = opt.config;
        this.patterns = opt.patterns;
        this.ngAppDistDir = this.outputDir;
        this.writeFile();
        await this.getBuildMeta();
    }
    // async build(): Promise<void> {
    //   console.log(`build()`);
    // }
    async getBuildMeta() {
        const indexPath = (0, path_1.join)(this.ngAppDistDir, 'index.html');
        const ngAppIndex = await (0, fs_extra_1.readFile)(indexPath, 'utf-8').catch((err) => {
            app_1.log.warn(`could not read index.html in ngAppDistDir: "${this.ngAppDistDir}"`);
        });
        if (!ngAppIndex) {
            return;
            // throw new Error(
            //   `Could not find "index.html" inside ngAppDistDir: "${this.ngAppDistDir}"`,
            // );
        }
        const base = '/knapsack-renderer-angular/';
        const $ = cheerio.load(ngAppIndex);
        const inlineFoots = [];
        $('script').each(function (i, el) {
            const elem = $(this);
            const src = elem.attr('src');
            elem.attr('src', `${base}${src}?cachebust=${Date.now()}`);
        });
        $('link[rel="stylesheet"]').each(function (i, el) {
            const elem = $(this);
            const href = elem.attr('href');
            elem.attr('href', `${base}${href}?cachebust=${Date.now()}`);
        });
        const scriptTags = $.html('script');
        const styleTags = $.html('link[rel="stylesheet"]');
        inlineFoots.push(scriptTags);
        inlineFoots.push(styleTags);
        this.extraHtmlTags = inlineFoots;
    }
    async render(opt) {
        const meta = {
            demo: opt.demo,
            patternId: opt.pattern.id,
            templateId: opt.template.id,
        };
        const usage = await this.getUsage(opt);
        const code = /* html */ `
    <div id="render-root" class="knapsack-pattern-direct-parent" data-dev-note="Knapsack Angular Template Wrapper">
      <ks-app-root> </ks-app-root>
      <script type="application/json" id="ks-angular-meta">${JSON.stringify(meta)}</script>
    </div>
    `;
        return {
            ok: true,
            html: KnapsackAngularRenderer.formatCode({
                code,
                language: 'html',
            }),
            usage,
        };
    }
    async getUsage(opt) {
        return '';
        // if (KnapsackAngularRenderer.isDataDemo(opt.demo)) {
        //   return JSON.stringify(opt.demo.data.props);
        // }
    }
    getFoot(opt) {
        return super.getFoot({
            ...opt,
            inlineFoot: `${opt.inlineFoot}${this.extraHtmlTags.join('\n')}`,
        });
    }
    async watch() {
        const watcher = chokidar_1.default.watch([this.ngAppDistDir], {
            ignoreInitial: true,
        });
        watcher.on('change', async (thePath) => {
            // log.info('ngAppDistDir changed...');
            await this.getBuildMeta();
            app_1.knapsackEvents.emitPatternTemplateChanged({
                path: thePath,
                renderer: this.id,
            });
            this.onChange();
        });
        app_1.knapsackEvents.onPatternTemplateChanged(({ path: thePath, renderer = '' }) => {
            this.getBuildMeta();
            app_1.log.verbose(`Angular pattern template changed handler path: ${thePath}`);
            /*
           Additions or changes to pattern templates using local client ui generate
           a PatternTemplateChanged event without a path and we need to trigger a
           ng build with updated templates.
    
           The ngAppDistDir watcher above generates a PatternTemplateChanged event with a path.
           Checking for this here to prevent an infinite loop between the two handlers.
          */
            if (renderer !== this.id) {
                app_1.log.verbose(`Angular renderer external pattern template change. Triggering rebuild.`);
                this.writeFile();
            }
            this.onChange();
        });
        app_1.knapsackEvents.onShutdown(() => watcher.close());
    }
    async getTemplateSuggestions({ newPath, }) {
        const { data: dataDir } = this.patterns.userConfig;
        const { allTemplateDemos, allTemplates } = this.getMyTemplates();
        const usedSuggestions = [
            ...allTemplateDemos,
            ...allTemplates,
        ].map(({ path, alias }) => {
            return {
                path,
                alias: alias || 'default',
            };
        });
        const allPaths = [
            ...new Set([newPath, ...usedSuggestions.map(({ path }) => path)]),
        ];
        const allSuggestions = await Promise.all(allPaths.map(async (path) => {
            if (!path)
                return [];
            try {
                const { exports, errorMsg } = await (0, file_utils_1.getJsExportNames)({
                    path,
                    resolveFromDir: dataDir,
                });
                if (errorMsg) {
                    throw new Error(errorMsg);
                }
                return (exports
                    // only allow names that start with a capital letter (i.e. React Component names)
                    // will exclude helper functions and custom hooks
                    .filter((e) => e === 'default' || (0, utils_1.isFirstLetterCapital)(e))
                    .map((name) => ({
                    alias: name,
                    path,
                })));
            }
            catch (e) {
                // only want this showing with verbose since template suggestions are not critical
                app_1.log.verbose(`Error getting import names for ${path}: ${e.message}`, null, this.logPrefix);
                return [];
            }
        })).then((results) => {
            // sadly `Promise.all` & `.flatMap` just don't work, so this:
            return results.flat();
        });
        const suggestions = allSuggestions.filter((s) => {
            return !usedSuggestions.find((us) => us.alias === s.alias && us.path === s.path);
        });
        return {
            suggestions,
        };
    }
}
exports.KnapsackAngularRenderer = KnapsackAngularRenderer;
//# sourceMappingURL=renderer-angular.js.map